@@ -20,6 +20,7 @@
 #include "sv_setrv.c.inc"
 
 #include "perl-additions.c.inc"
+#include "forbid_outofblock_ops.c.inc"
 #include "force_list_keeping_pushmark.c.inc"
 #include "optree-additions.c.inc"
 #include "newOP_CUSTOM.c.inc"
@@ -140,10 +141,10 @@ void ObjectPad_mop_class_apply_attribute(pTHX_ ClassMeta *classmeta, const char
     if((reg->funcs->flags & OBJECTPAD_FLAG_ATTR_MUST_VALUE) && !value)
       croak("Attribute :%s requires a value", name);
 
-    SV *hookdata = value;
+    SV *attrdata = value;
 
     if(reg->funcs->apply) {
-      if(!(*reg->funcs->apply)(aTHX_ classmeta, value, &hookdata, reg->funcdata))
+      if(!(*reg->funcs->apply)(aTHX_ classmeta, value, &attrdata, reg->funcdata))
         return;
     }
 
@@ -156,12 +157,12 @@ void ObjectPad_mop_class_apply_attribute(pTHX_ ClassMeta *classmeta, const char
     *hook = (struct ClassHook){
       .funcs    = reg->funcs,
       .funcdata = reg->funcdata,
-      .hookdata = hookdata,
+      .attrdata = attrdata,
     };
 
     av_push(classmeta->hooks, (SV *)hook);
 
-    if(value && value != hookdata)
+    if(value && value != attrdata)
       SvREFCNT_dec(value);
 
     return;
@@ -170,7 +171,79 @@ void ObjectPad_mop_class_apply_attribute(pTHX_ ClassMeta *classmeta, const char
   croak("Unrecognised class attribute :%s", name);
 }
 
-/* TODO: get attribute */
+static ClassAttributeRegistration *get_active_registration(pTHX_ const char *name)
+{
+  COPHH *cophh = CopHINTHASH_get(PL_curcop);
+
+  for(ClassAttributeRegistration *reg = classattrs; reg; reg = reg->next) {
+    if(!strEQ(name, reg->name))
+      continue;
+
+    if(reg->funcs->permit_hintkey &&
+        !cophh_fetch_pvn(cophh, reg->funcs->permit_hintkey, reg->permit_hintkeylen, 0, 0))
+      continue;
+
+    return reg;
+  }
+
+  return NULL;
+}
+
+struct ClassHook *ObjectPad_mop_class_get_attribute(pTHX_ ClassMeta *classmeta, const char *name)
+{
+  /* First, work out what hookfuncs the name maps to */
+  ClassAttributeRegistration *reg = get_active_registration(aTHX_ name);
+
+  if(!reg)
+    return NULL;
+
+  /* Now lets see if classmeta has one */
+
+  if(!classmeta->hooks)
+    return NULL;
+
+  U32 hooki;
+  for(hooki = 0; hooki < av_count(classmeta->hooks); hooki++) {
+    struct ClassHook *hook = (struct ClassHook *)AvARRAY(classmeta->hooks)[hooki];
+
+    if(hook->funcs == reg->funcs)
+      return hook;
+  }
+
+  return NULL;
+}
+
+AV *ObjectPad_mop_class_get_attribute_values(pTHX_ ClassMeta *classmeta, const char *name)
+{
+  /* First, work out what hookfuncs the name maps to */
+
+  ClassAttributeRegistration *reg = get_active_registration(aTHX_ name);
+
+  if(!reg)
+    return NULL;
+
+  /* Now lets see if classmeta has one */
+
+  if(!classmeta->hooks)
+    return NULL;
+
+  AV *ret = NULL;
+
+  U32 hooki;
+  for(hooki = 0; hooki < av_count(classmeta->hooks); hooki++) {
+    struct ClassHook *hook = (struct ClassHook *)AvARRAY(classmeta->hooks)[hooki];
+
+    if(hook->funcs != reg->funcs)
+      continue;
+
+    if(!ret)
+      ret = newAV();
+
+    av_push(ret, newSVsv(hook->attrdata));
+  }
+
+  return ret;
+}
 
 ClassMeta *ObjectPad_mop_get_class_for_stash(pTHX_ HV *stash)
 {
@@ -752,6 +825,9 @@ void ObjectPad_mop_class_add_ADJUST(pTHX_ ClassMeta *meta, CV *cv)
 {
   if(meta->sealed)
     croak("Cannot add an ADJUST(PARAMS) block to an already-sealed class");
+
+  warn_outofblock_ops(CvROOT(cv), "Using %s to leave an ADJUST block is discouraged and will be removed in a later version");
+
   if(!meta->adjustblocks)
     meta->adjustblocks = newAV();
 
@@ -964,7 +1040,7 @@ static struct FieldHook *S_embed_fieldhook(pTHX_ struct FieldHook *roleh, FIELDO
     .fieldix   = roleh->fieldix + offset,
     .fieldmeta = roleh->fieldmeta,
     .funcs     = roleh->funcs,
-    .hookdata  = roleh->hookdata,
+    .attrdata  = roleh->attrdata,
   };
 
   return classh;
@@ -1033,14 +1109,14 @@ static void S_mop_class_apply_role(pTHX_ RoleEmbedding *embedding)
     }
   }
 
-  if(rolemeta->fieldhooks_initfield) {
-    if(!classmeta->fieldhooks_initfield)
-      classmeta->fieldhooks_initfield = newAV();
+  if(rolemeta->fieldhooks_makefield) {
+    if(!classmeta->fieldhooks_makefield)
+      classmeta->fieldhooks_makefield = newAV();
 
     U32 i;
-    for(i = 0; i < av_count(rolemeta->fieldhooks_initfield); i++) {
-      struct FieldHook *roleh = (struct FieldHook *)AvARRAY(rolemeta->fieldhooks_initfield)[i];
-      av_push(classmeta->fieldhooks_initfield, (SV *)embed_fieldhook(roleh, embedding->offset));
+    for(i = 0; i < av_count(rolemeta->fieldhooks_makefield); i++) {
+      struct FieldHook *roleh = (struct FieldHook *)AvARRAY(rolemeta->fieldhooks_makefield)[i];
+      av_push(classmeta->fieldhooks_makefield, (SV *)embed_fieldhook(roleh, embedding->offset));
     }
   }
 
@@ -1282,9 +1358,9 @@ void ObjectPad_mop_class_seal(pTHX_ ClassMeta *meta)
       for(hooki = 0; fieldmeta->hooks && hooki < av_count(fieldmeta->hooks); hooki++) {
         struct FieldHook *h = (struct FieldHook *)AvARRAY(fieldmeta->hooks)[hooki];
 
-        if(*h->funcs->post_initfield) {
-          if(!meta->fieldhooks_initfield)
-            meta->fieldhooks_initfield = newAV();
+        if(*h->funcs->post_makefield) {
+          if(!meta->fieldhooks_makefield)
+            meta->fieldhooks_makefield = newAV();
 
           struct FieldHook *fasth;
           Newx(fasth, 1, struct FieldHook);
@@ -1294,10 +1370,10 @@ void ObjectPad_mop_class_seal(pTHX_ ClassMeta *meta)
             .fieldmeta = fieldmeta,
             .funcs     = h->funcs,
             .funcdata  = h->funcdata,
-            .hookdata  = h->hookdata,
+            .attrdata  = h->attrdata,
           };
 
-          av_push(meta->fieldhooks_initfield, (SV *)fasth);
+          av_push(meta->fieldhooks_makefield, (SV *)fasth);
         }
 
         if(*h->funcs->post_construct) {
@@ -1312,7 +1388,7 @@ void ObjectPad_mop_class_seal(pTHX_ ClassMeta *meta)
             .fieldmeta = fieldmeta,
             .funcs     = h->funcs,
             .funcdata  = h->funcdata,
-            .hookdata  = h->hookdata,
+            .attrdata  = h->attrdata,
           };
 
           av_push(meta->fieldhooks_construct, (SV *)fasth);
@@ -1518,7 +1594,7 @@ XS_INTERNAL(injected_constructor)
 
   SV **fieldsvs = AvARRAY(backingav);
 
-  if(meta->fieldhooks_initfield || meta->fieldhooks_construct) {
+  if(meta->fieldhooks_makefield || meta->fieldhooks_construct) {
     /* We need to set up a fake pad so these hooks can still get PADIX_SELF / PADIX_SLOTS */
 
     /* This MVP is just sufficient enough to let PAD_SVl(PADIX_SELF) work */
@@ -1530,17 +1606,17 @@ XS_INTERNAL(injected_constructor)
     PAD_SVl(PADIX_SLOTS) = (SV *)backingav;
   }
 
-  if(meta->fieldhooks_initfield) {
+  if(meta->fieldhooks_makefield) {
     DEBUG_SET_CURCOP_LINE(__LINE__);
 
-    AV *fieldhooks = meta->fieldhooks_initfield;
+    AV *fieldhooks = meta->fieldhooks_makefield;
 
     U32 i;
     for(i = 0; i < av_count(fieldhooks); i++) {
       struct FieldHook *h = (struct FieldHook *)AvARRAY(fieldhooks)[i];
       FIELDOFFSET fieldix = h->fieldix;
 
-      (*h->funcs->post_initfield)(aTHX_ h->fieldmeta, h->hookdata, h->funcdata, fieldsvs[fieldix]);
+      (*h->funcs->post_makefield)(aTHX_ h->fieldmeta, h->attrdata, h->funcdata, fieldsvs[fieldix]);
     }
   }
 
@@ -1677,7 +1753,7 @@ XS_INTERNAL(injected_constructor)
       struct FieldHook *h = (struct FieldHook *)AvARRAY(fieldhooks)[i];
       FIELDOFFSET fieldix = h->fieldix;
 
-      (*h->funcs->post_construct)(aTHX_ h->fieldmeta, h->hookdata, h->funcdata, fieldsvs[fieldix]);
+      (*h->funcs->post_construct)(aTHX_ h->fieldmeta, h->attrdata, h->funcdata, fieldsvs[fieldix]);
     }
   }
 
@@ -1947,11 +2023,11 @@ void ObjectPad_mop_class_set_superclass(pTHX_ ClassMeta *meta, SV *superclassnam
       av_push_from_av_noinc(meta->adjustblocks, supermeta->adjustblocks);
     }
 
-    if(supermeta->fieldhooks_initfield) {
-      if(!meta->fieldhooks_initfield)
-        meta->fieldhooks_initfield = newAV();
+    if(supermeta->fieldhooks_makefield) {
+      if(!meta->fieldhooks_makefield)
+        meta->fieldhooks_makefield = newAV();
 
-      av_push_from_av_noinc(meta->fieldhooks_initfield, supermeta->fieldhooks_initfield);
+      av_push_from_av_noinc(meta->fieldhooks_makefield, supermeta->fieldhooks_makefield);
     }
 
     if(supermeta->fieldhooks_construct) {
@@ -2075,7 +2151,7 @@ static const char *S_split_package_ver(pTHX_ SV *value, SV *pkgname, SV *pkgvers
 
 /* :isa */
 
-static bool classhook_isa_apply(pTHX_ ClassMeta *classmeta, SV *value, SV **hookdata_ptr, void *_funcdata)
+static bool classhook_isa_apply(pTHX_ ClassMeta *classmeta, SV *value, SV **attrdata, void *_funcdata)
 {
   SV *superclassname = newSV(0), *superclassver = newSV(0);
   SAVEFREESV(superclassname);
@@ -2116,7 +2192,7 @@ static const struct ClassHookFuncs classhooks_isa = {
 
 /* :does */
 
-static bool classhook_does_apply(pTHX_ ClassMeta *classmeta, SV *value, SV **hookdata_ptr, void *_funcdata)
+static bool classhook_does_apply(pTHX_ ClassMeta *classmeta, SV *value, SV **attrdata, void *_funcdata)
 {
   SV *rolename = newSV(0), *rolever = newSV(0);
   SAVEFREESV(rolename);
@@ -2140,7 +2216,7 @@ static const struct ClassHookFuncs classhooks_does = {
 
 /* :repr */
 
-static bool classhook_repr_apply(pTHX_ ClassMeta *classmeta, SV *value, SV **hookdata_ptr, void *_funcdata)
+static bool classhook_repr_apply(pTHX_ ClassMeta *classmeta, SV *value, SV **attrdata, void *_funcdata)
 {
   char *val = SvPV_nolen(value); /* all comparisons are ASCII */
 
@@ -2172,7 +2248,7 @@ static const struct ClassHookFuncs classhooks_repr = {
 
 /* :compat */
 
-static bool classhook_compat_apply(pTHX_ ClassMeta *classmeta, SV *value, SV **hookdata_ptr, void *_funcdata)
+static bool classhook_compat_apply(pTHX_ ClassMeta *classmeta, SV *value, SV **attrdata, void *_funcdata)
 {
   if(strEQ(SvPV_nolen(value), "invokable")) {
     if(classmeta->type != METATYPE_ROLE)
@@ -2194,7 +2270,7 @@ static const struct ClassHookFuncs classhooks_compat = {
 
 /* :strict */
 
-static bool classhook_strict_apply(pTHX_ ClassMeta *classmeta, SV *value, SV **hookdata_ptr, void *_funcdata)
+static bool classhook_strict_apply(pTHX_ ClassMeta *classmeta, SV *value, SV **attrdata_ptr, void *_funcdata)
 {
   if(strEQ(SvPV_nolen(value), "params"))
     classmeta->strict_params = TRUE;
